---
elm:
  dependencies:
    avh4/elm-fifo: latest
    elm/browser: latest
    elm/core: latest
    elm/html: latest
    elm/json: latest
    elm/parser: latest
    elm/regex: latest
    elm/time: latest
    elm/url: latest
    elm/virtual-dom: latest
    elm-community/graph: latest
    elm-community/intdict: latest
    elm-community/list-extra: latest
    elm-community/maybe-extra: latest
    elm-community/string-extra: latest

  source-directories:
    - ../src
---

# Problema del robo

> Fuente: Ejercios de Lógica Computacional
> (https://www.cs.us.es/~fsancho/?p=logica-informatica-2020-21)

Alberto, Berta y Carlos son los tres sospechosos de un robo. Se les interroga por separado y éstas son sus declaraciones:

- _Alberto_: Berta es culpable y Carlos es inocente

- _Berta_: Si Alberto es culpable, Carlos también.

- _Carlos_: Yo soy inocente, pero al menos uno de los otros dos es culpable

¿Son consistentes las declaraciones?¿quién es inocente y quién culpable si todos dijeron la verdad?¿y si únicamente lo hicieron las personas inocentes?

## Solución

### Parte I

Sean $a, b, c$ variables proposicionales que indican si Alberto, Berta y Carlos son culpables (V) o si son inocentes (F). De forma que los testimonios podrían expresarse como:

$$ \{b \wedge \neg c, \, a \rightarrow c, \, \neg c \wedge (a \vee b) \} $$

Ahora, determinar la consistencia del conjunto es equivalente a determinar la satisfactibilidad de la conjunción de las fórmulas. Para lo que utilizaremos la FND. Haciendo uso de LogicUS:

```elm {l id="imports"}
import LogicUS.PL.SintaxSemantics exposing (..)
import LogicUS.PL.NormalForms exposing (..)

fplRead : String -> FormulaPL
fplRead = fplReadExtraction << fplReadFromString
```

Definimos las fórmulas:

```elm {l context="1" follows="imports"}
f1 : FormulaPL
f1 = fplRead "b & ¬ c"

f2 : FormulaPL
f2 = fplRead "a -> c"

f3 : FormulaPL
f3 = fplRead "¬c & (a | b)"

```

^^^elm{m=("$$ " ++ (String.join "," <| List.map fplToMathString [f1,f2,f3]) ++ " $$") context="1"}^^^

Entonces ahora para determinar la satisfactibilidad podemos hacerlo a través de la función `fplSatisfiabilityDNF` y la conjunción de las fórmulas del conjunto:

```elm {l context="1"}
g : FormulaPL
g = splConjunction [f1, f2, f3]
```

que corresponde a:

^^^elm{m=("$$ " ++ (fplToMathString g) ++ " $$") context="1"}^^^

De forma que veamos si $g$ es satisfactible. Vamos a hacerlo pasos a paso, en vez de utilizar directamente `fplSatisfiabilityDNF`:

1. Hallamos una FND de $g$:

```elm {l context="1"}
g1 : FormulaPL
g1 = fplToDNF g
```

^^^elm{m=("$$ " ++ (fplToMathString g1) ++ " $$") context="1"}^^^

<br> O como conjunto de conjuntos de literales:

```elm {l context="1"}
g1_ls : Maybe (List SetPL)
g1_ls = dnfAsLiteralSets g1
```

^^^elm{m=("$$ " ++ (String.join "," <| List.map splToMathString2 <| Maybe.withDefault [] g1_ls) ++ " $$") context="1"}^^^

<br> De forma que la fórmula, y por ende, el conjunto es consistente ya que existe al menos un conjunto (de hecho sólo existe 1) que no posee literales complementarios. Obsérvese también que dado que se ha obtenido un único conjunto sin literales complementarios en el que además aparecen todos los símbolos proposicionales entonces éste proporciona el único modelo para la fórmula. De forma que es fácil deducir que, si todos dijeron la verdad, la culpable fue Berta.

### Parte II

Ahora, si solo dijeron la verdad los inocentes entonces el problema podríamos plantearlo como:

$$ \{(b \wedge \neg c) \leftrightarrow \neg a, \, (a \rightarrow c) \leftrightarrow \neg b, \, (\neg c \wedge (a \vee b))\leftrightarrow \neg c \} $$

Ahora, determinar la consistencia del conjunto es equivalente a determinar la satisfactibilidad de la conjunción de las fórmulas. Para lo que utilizaremos la FND. Haciendo uso de LogicUS:

```elm {l context="2" follows="imports"}

f1 : FormulaPL
f1 = fplRead "(b & ¬c) <-> ¬a"

f2 : FormulaPL
f2 = fplRead "(a -> c) <-> ¬b"

f3 : FormulaPL
f3 = fplRead "(¬c & (a | b)) <-> ¬c"

```

^^^elm{m=("$$ " ++ (String.join "," <| List.map fplToMathString [f1,f2,f3]) ++ " $$") context="2"}^^^

<br> De forma que comprobamos que el conjunto es consistente:

```elm {l r="gIsSAT" context="2"}
g : FormulaPL
g = splConjunction [f1,f2,f3]

gIsSAT : Bool
gIsSAT = fplSatisfiabilityDNF g

```

<br> Y sus modelos corresponden a

```elm {l context="2"}
gModels : List Interpretation
gModels =  fplModelsDNF g

```

^^^elm{m=("$$ " ++ (interpretationsToMathString gModels [("a",[]),("b",[]),("c",[])]) ++ " $$") context="2"}^^^

<br> De manera que bajo el segundo supuesto, los culpables se habrían aliado para delatar a Berta.
